<?php

namespace Ecommerce\Controllers;

use Doctrine\ORM\EntityManager;

class ExportaXmlMercadoEletronico 
{
    protected $entityManager;
    protected $connection;
    
    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
	}

    public function exportarXml ($paramDadosAf, $quebraXmlAf)
    {
        $afs = array();
        
        foreach ($paramDadosAf as $param) {
            $af = $this->retornaDadosAfME($param);
            $afs = array_merge($afs, $af);
        }

        $xml = $this->converteArrayXml($afs, $quebraXmlAf);
        
        return $xml;
    }
    
    private function substituiCaracteres($valor)
	{
	    $retirar = array('<', '>', '"', '\'', '&lt;', '&gt;', '“', '”', '&nbsp;', '&amp;');
        $substituir = array('', '', '', '', '', '', '', '', ' ', '&');
	    
	    $valor = strip_tags($valor);
        
        $valor = str_replace($retirar, $substituir, $valor);
        
        if (mb_detect_encoding($valor) != 'UTF-8') {
            $valor = utf8_encode($valor);
        }
        
        return $valor;
	}
    
    private function converteArrayXml($afs, $quebraXmlAf)
    {
        $loteAf = array();
        
        $dom = new \DOMDocument("1.0", "UTF-8");

        $dom->preserveWhiteSpace = false;
        $dom->formatOutput = true;
        
        $elementoLoteAf = $dom->createElement("MEBrokerXMLMessage");
        
        $this->criaElementosEstruturaXml($dom, $elementoHeaders, $elementoMensagem, $elementoMsgFornecedor, $elementoItens, $elementoAtributos, $afs[0]["PEDIDO"]);
        $this->iniciaMarcadoresMontagemXml($afAnterior, $preencherCabecalho, $numeroItem, $afs[0]['PEDIDO']);
        
        foreach ($afs as $af) {
            if ($af['PEDIDO'] !== $afAnterior) {
                
                $this->preencheElementosEstruturaXml($elementoLoteAf, $elementoHeaders, $elementoMensagem, $elementoMsgFornecedor, $elementoItens, $elementoAtributos);
                $this->quebraArquivoAF($quebraXmlAf, $dom, $elementoLoteAf, $loteAf, $afAnterior);
                $this->criaElementosEstruturaXml($dom, $elementoHeaders, $elementoMensagem, $elementoMsgFornecedor, $elementoItens, $elementoAtributos, $af["PEDIDO"]);
                $this->iniciaMarcadoresMontagemXml($afAnterior, $preencherCabecalho, $numeroItem, $af['PEDIDO']);
            
            }
            
            $numero = $dom->createElement("NUMEROITEM", $numeroItem ++);
            $numero->setAttribute("Tipo", $this->retornaTipoColuna($numeroItem));
            
            $elementoItem = $dom->createElement("Item");
            $elementoItem->appendChild($numero);
            
            foreach ($af as $nome => $valor) {
                $valor = $this->substituiCaracteres($valor);
                
                switch ($nome) {
                    case strpos($nome, 'MSGFORN_'):
                        $this->preencheElementoMsgFornecedor($dom, $elementoMsgFornecedor, $preencherCabecalho, $nome, $valor);
                        break;
                        
                    case strpos($nome, 'ITENS_'):
                        $this->preencheElementoItem($dom, $elementoItem, $nome, $valor);
                        break;    
                        
                    case strpos($nome, 'ATRIBUTOS_'):
                        $this->preencheElementoAtributos($dom, $elementoAtributos, $preencherCabecalho, $nome, $valor);
                        break;
                    
                    default:
                        $this->preencheElementoMensagem($dom, $elementoMensagem, $preencherCabecalho, $nome, $valor);
                        break;
                }
            }
            
            $elementoItens->appendChild($elementoItem);
            $preencherCabecalho = false;
        }
        
        $this->preencheElementosEstruturaXml($elementoLoteAf, $elementoHeaders, $elementoMensagem, $elementoMsgFornecedor, $elementoItens, $elementoAtributos);
        
        $dom->appendChild($elementoLoteAf);
        
        $nomeIndice = $quebraXmlAf ? $af['PEDIDO'] : 'RelatorioAF';
        $loteAf[$nomeIndice] = $dom->saveXml();

        return $loteAf;
    }
    
    private function retornaTipoColuna($valor)
    {
        $listaTipos = array(
            "string" => "String",
            "double" => "Float",
            "integer" => "Integer"
        );
        
        $tipo = gettype($valor);

        return $listaTipos[$tipo];
    }
    
    private function criaElementosEstruturaXml($dom, &$elementoHeaders, &$elementoMensagem, &$elementoMsgFornecedor, &$elementoItens, &$elementoAtributos, $pedido)
    {
        $elementoHeaders = $dom->createElement("MEHeaders");
        $elementoHeaders->appendChild($dom->createElement("TipoMensagem", "Pedido"));
        $elementoHeaders->appendChild($dom->createElement("ID", $pedido));
        
        $elementoMensagem = $dom->createElement("Mensagem");
        
        $elementoMsgFornecedor = $dom->createElement("MsgFornecedor");
        $elementoMsgFornecedor->setAttribute("NodeType", "Objeto");
        
        $elementoItens = $dom->createElement("Itens");
        $elementoItens->setAttribute("NodeType", "Itens");
        
        $elementoAtributos = $dom->createElement("Atributos");
        $elementoAtributos->setAttribute("NodeType", "Atributos");
    }
    
    private function preencheElementosEstruturaXml(&$elementoLoteAf, &$elementoHeaders, &$elementoMensagem, &$elementoMsgFornecedor, &$elementoItens, &$elementoAtributos)
    {
        $elementoMensagem->appendChild($elementoMsgFornecedor);
        $elementoMensagem->appendChild($elementoItens);
        $elementoMensagem->appendChild($elementoAtributos);
        
        $elementoLoteAf->appendChild($elementoHeaders);
        $elementoLoteAf->appendChild($elementoMensagem);
    }
    
    private function preencheElementoMsgFornecedor($dom, &$elementoMsgFornecedor, $preencherCabecalho, $nome, $valor)
    {
        if ($preencherCabecalho) {
            $nome = str_replace('MSGFORN_', '', $nome);
            
            $msgFornecedor = $dom->createElement($nome, htmlspecialchars($valor));
            
            if ($valor) {
                $msgFornecedor->setAttribute("Tipo", $this->retornaTipoColuna($valor));
            }
            
            $elementoMsgFornecedor->appendChild($msgFornecedor);
        }
    }
    
    private function preencheElementoItem($dom, &$elementoItem, $nome, $valor)
    {
        $nome = str_replace('ITENS_', '', $nome);
        $item = $dom->createElement($nome, htmlspecialchars($valor));
        
        if ($valor) {
            $item->setAttribute("Tipo", $this->retornaTipoColuna($valor));
        }
        
        $elementoItem->appendChild($item);
    }
    
    private function preencheElementoAtributos($dom, &$elementoAtributos, $preencherCabecalho, $nome, $valor)
    {
        if ($preencherCabecalho) {
            $nome = str_replace('ATRIBUTOS_', '', $nome);
            
            $atributos = $dom->createElement("Atributo");
            $atributos->setAttribute("NomeAtributo", $nome);
            $atributos->setAttribute("ValorAtributo", htmlspecialchars($valor));
            
            $elementoAtributos->appendChild($atributos);
        }
    }
    
    private function preencheElementoMensagem($dom, &$elementoMensagem, $preencherCabecalho, $nome, $valor)
    {
        if ($preencherCabecalho) {
            $mensagem = $dom->createElement($nome, htmlspecialchars($valor));
            
            if ($valor) {
                $mensagem->setAttribute("Tipo", $this->retornaTipoColuna($valor));
            }
            
            $elementoMensagem->appendChild($mensagem);
        }
    }
    
    private function iniciaMarcadoresMontagemXml(&$afAnterior, &$preencherCabecalho, &$numeroItem, $pedido)
    {
        $afAnterior = $pedido;
        $preencherCabecalho = true;
        $numeroItem = 1;
    }
    
    private function quebraArquivoAF($quebraXmlAf, &$dom, &$elementoLoteAf, &$loteAf, $pedido)
    {
        if ($quebraXmlAf) {
    
            $dom->appendChild($elementoLoteAf);
            $loteAf[$pedido] = $dom->saveXml();
            
            $dom->removeChild($elementoLoteAf);
            $elementoLoteAf = $dom->createElement("MEBrokerXMLMessage");
            
        }
    }
    
    public function retornaDadosAfME($row)
    {
        $query = "SELECT 
                        '' AS IDINTEGRACAO,
                        '' AS CONTRATO,
                        AF.NRAUTOFORN AS PEDIDO,
                        DECODE(EA.IDTCLASSENVAFOR, 'P', 'S', 'N') AS ALTERACAO,
                        FN.CDFORNECED AS FORNECEDORCLIENTE,
                        PARCMPSUP.DSOBSFILIAL AS OBSCLI,
                        FN.NMFANTFORN AS NOME,
                        FN.NMRAZSOCFORN AS RAZAO,
                        FN.NMCONTFORN AS CONTATO,
                        FN.DSENDEFORN AS ENDERECO,
                        FN.NMMUNICIPIO AS CIDADE,
                        FN.SGESTADO AS ESTADO,
                        FN.CDPAIS AS PAIS,
                        FN.NRCEPFORN AS CEP,
                        FN.NRTELEFORN AS FONE,
                        FN.NRFAXFORN AS FAX,
                        FN.DSEMAILFORN AS EMAIL,
                        FN.NRINSJURFORN AS CNPJ,
                        OPLANC.DSEMAILOPER AS EMAIL_COMPRADOR,
                        CASE IDTCLASSENVAFOR WHEN 'N'   THEN 'NORMAL'
                            WHEN 'E'   THEN 'EXTRA'
                            WHEN 'A'   THEN 'ANTECIPAÇÃO'
                            WHEN 'C'   THEN 'COMPLEMENTO'
                            WHEN 'I'   THEN 'INAUGURAÇÃO'
                            WHEN 'P'   THEN 'PLANILHA DE ALTERAÇÃO'
                        END AS TIPOPEDIDO,
                        AF.CDFILDEST AS CENTRO,
                        FD.NRINSJURFILI AS CNPJLOCALENTREGA,
                        OPLANC.NMOPERADOR AS NOMECOMPRADOR,
                        DECODE(PARAMFILIAL.CDTIPOFATUR, '003', CLIENTE.NRINSJURCLIE, FATURAMENTO.NRINSJURIEST) AS CNPJLOCALFATURAMENTO,
                        DECODE(PARAMFILIAL.CDTIPOFATUR, '003', ENDECLIE.DSENDECLIE, FATURAMENTO.DSENDEIEST) AS ENDERECOLOCALFATURAMENTO,
                        DECODE(PARAMFILIAL.CDTIPOFATUR, '003', ENDECLIE.DSCOMPENDCLI, FATURAMENTO.DSCOMPENDIES) AS COMPLEMENTOLOCALFATURAMENTO,
                        DECODE(PARAMFILIAL.CDTIPOFATUR, '003', MUNICLIE.NMMUNICIPIO, FATURAMENTO.NMMUNICIPIO) AS CIDADELOCALFATURAMENTO,
                        DECODE(PARAMFILIAL.CDTIPOFATUR, '003', ENDECLIE.NRCEPCLIE, FATURAMENTO.NRCEPIEST) AS CEPLOCALFATURAMENTO,
                        DECODE(PARAMFILIAL.CDTIPOFATUR, '003', ENDECLIE.NMBAIRCLIE, FATURAMENTO.NMBAIRIEST) AS BAIRROLOCALFATURAMENTO,
                        DECODE(PARAMFILIAL.CDTIPOFATUR, '003', ENDECLIE.SGESTADO, FATURAMENTO.SGESTADO) AS ESTADOLOCALFATURAMENTO,
                        FD.DSENDEFILI AS ENDERECOLOCALENTREGA,
                        FD.DSCOMPENDFIL AS COMPLEMENTOLOCALENTREGA,
                        FD.NMMUNICIFIL AS CIDADELOCALENTREGA,
                        FD.NRCEPFILI AS CEPLOCALENTREGA,
                        FD.NMCONTFILI AS CONTATOLOCALENTREGA,
                        FD.NRTELEFILI AS TELEFONELOCALENTREGA,
                        FD.NMBAIRFILI AS BAIRROLOCALENTREGA,
                        FD.SGESTADOFIL AS ESTADOLOCALENTREGA,
                        DECODE(AF.IDTIPOEMITAF, '3', CLIENTE.NRINSJURCLIE, EMPRESA.NRINSJUREMPR) AS CNPJLOCALCOBRANCA,
                        DECODE(AF.IDTIPOEMITAF, '3', ENDECLIE.DSENDECLIE, EMPRESA.DSENDEEMPR) AS ENDERECOLOCALCOBRANCA,
                        DECODE(AF.IDTIPOEMITAF, '3', MUNICLIE.NMMUNICIPIO, MUNICIPIO.NMMUNICIPIO) AS CIDADELOCALCOBRANCA,
                        DECODE(AF.IDTIPOEMITAF, '3', ENDECLIE.NRCEPCLIE, EMPRESA.NRCEPEMPR) AS CEPLOCALCOBRANCA,
                        DECODE(AF.IDTIPOEMITAF, '3', ENDECLIE.NMCONTCLIE, EMPRESA.NMCONTEMPR) AS CONTATOLOCALCOBRANCA,
                        DECODE(AF.IDTIPOEMITAF, '3', ENDECLIE.NRTELECLIE, EMPRESA.NRTELEFOEMPR) AS TELEFONELOCALCOBRANCA,
                        DECODE(AF.IDTIPOEMITAF, '3', ENDECLIE.NMBAIRCLIE, EMPRESA.NMBAIREMPR) AS BAIRROLOCALCOBRANCA,
                        DECODE(AF.IDTIPOEMITAF, '3', ENDECLIE.SGESTADO, EMPRESA.SGESTADO) AS ESTADOLOCALCOBRANCA,
                        FN.CDFORNECED AS MsgForn_FORNECEDORCLIENTE,
                        FN.NMFANTFORN AS MsgForn_NOME,
                        FN.NMRAZSOCFORN AS MsgForn_RAZAO,
                        FN.NMCONTFORN AS MsgForn_CONTATO,
                        FN.DSENDEFORN AS MsgForn_ENDERECO,
                        FN.DSCOMPENDFOR AS MsgForn_ENDERECOCOMPLEMENTO,
                        FN.NMMUNICIPIO AS MsgForn_CIDADE,
                        FN.SGESTADO AS MsgForn_ESTADO,
                        FN.CDPAIS AS MsgForn_PAIS,
                        FN.NRCEPFORN AS MsgForn_CEP,    
                        FN.NRCELULARFOR AS MsgForn_CELULAR,
                        FN.NMCONTFORN AS MsgForn_CONTATO2,
                        FN.DSEMAILFORN AS MsgForn_EMAIL2,
                        FN.NMBAIRFORN AS MsgForn_BAIRRO,
                        FN.NRTELEFORN AS MsgForn_TELEFONE,
                        FN.NRFAXFORN AS MsgForn_FAX,
                        FN.DSEMAILFORN AS MsgForn_EMAIL,
                        FN.NRINSJURFORN AS MsgForn_CNPJ,
                        ITEMAUTF.QTPROGENTAF AS Itens_QUANTIDADE,
                        ITEMAUTF.VRPREUNIAF AS Itens_PRECO,    
                        ITEMAUTF.VRPREUNIAF - NVL(ITEMAUTF.VRDESCITMAF, 0) AS Itens_PRECOLIQUIDO,
                        DECODE(IDTIPOLANITM, 'M', '*', ' ') || ' ' || PRODUTO.NMPRODUTO AS Itens_DESCRICAO,
                        PRODUTO.SGUNIDADE AS Itens_UNIDADE,
                        MARCA.CDMARCA AS Itens_CODMARCA,
                        MARCA.NMMARCA AS Itens_MARCA,
                        ITEMAUTF.DTPROGENTAF AS Itens_DATAENTREGA,
                        ITEMAUTF.CDPRODUTO AS Itens_CODPRODUTO,
                        PARCMPSUP.DSPERIENTFIL AS Atributos_DSPERIENTFIL,
                        PARAMGERALECOMMERCE.TXMSGFORNECEDOR AS CBC
                    FROM AUTOFORNE AF, CLIENFILIAL, ENDECLIE, CLIENTE, EMPRESA, MUNICIPIO MUNICLIE,
                         MUNICIPIO, ITEMAUTF, PRODUTO, MARCA, MARCA SEGMARCA, PARCMPSUP, PARAMFILIAL,
                         OPERADOR OP, OPERADOR OPLANC, ENVIOAFFORN EA, PARAMGERALECOMMERCE,
                         (SELECT FILIAL.CDFILIAL, FILIAL.NMFILIAL, FILIAL.NRINSJURFILI, FILIAL.IDTPIJURFILI,
                                 FILIAL.NMRAZSOCFILI, MUNIFIL.NMMUNICIPIO AS NMMUNICIFIL, ENDEFILI.DSENDEFILI,
                                 ENDEFILI.NMBAIRFILI, ENDEFILI.SGESTADO AS SGESTADOFIL, ENDEFILI.NRCEPFILI, ENDEFILI.DSCOMPENDFIL,
                                 ENDEFILI.NRTELEFILI, ENDEFILI.NMCONTFILI, NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL) AS CDFILCENTCMP
                          FROM FILIAL, ENDEFILI, MUNICIPIO MUNIFIL
                          WHERE 
                                (FILIAL.CDFILIAL       = ENDEFILI.CDFILIAL  ) AND
                                (ENDEFILI.IDTPENDEFILI = DECODE((SELECT IDTPENDEFILI FROM ENDEFILI WHERE IDTPENDEFILI = 'T' AND CDFILIAL = FILIAL.CDFILIAL),'T','T','P')) AND
                                (ENDEFILI.CDPAIS       = MUNIFIL.CDPAIS     ) AND
                                (ENDEFILI.SGESTADO     = MUNIFIL.SGESTADO   ) AND
                                (ENDEFILI.CDMUNICIPIO  = MUNIFIL.CDMUNICIPIO)
                         ) FD,
                         (SELECT F.CDFORNECED, F.IDTPIJURFORN, F.NRINSJURFORN, F.NMRAZSOCFORN, F.NMFANTFORN, F.CDFORNERELACAF,
                                 E.DSENDEFORN, E.NMCONTFORN, E.NRTELEFORN, F.DSEMAILCOT, E.CDMUNICIPIO, M.NMMUNICIPIO,
                                 E.SGESTADO, E.CDPAIS, E.NRCEPFORN, E.NRFAXFORN, E.DSEMAILFORN, E.DSCOMPENDFOR,
                                 E.NRCELULARFOR, E.NMBAIRFORN
                          FROM FORNECEDOR F, ENDEFORN E, MUNICIPIO M
                          WHERE 
                                (E.SGESTADO     = M.SGESTADO )    AND
                                (E.CDMUNICIPIO  = M.CDMUNICIPIO ) AND
                                (E.CDPAIS       = M.CDPAIS )      AND
                                (F.CDFORNECED   = E.CDFORNECED )  AND
                                (E.IDTPENDEFORN = 'P'          )  AND
                                (F.CDFORNECED  = :P_CDFORNECED)
                         ) FN,
                         (SELECT I.NMREPRINSEST, I.DSENDEIEST, I.NRINSJURIEST, I.IDTPIJURIEST, 
                                I.NRCEPIEST, I.CDMUNICIPIO, M.NMMUNICIPIO , I.CDINSCESTA, F.CDFILIAL, I.NMBAIRIEST,
                                I.SGESTADO, I.DSCOMPENDIES
                           FROM 
                               FILIAL F, INSCRESTAD I, MUNICIPIO M, PAIS P, ESTADO E
                          WHERE
                               F.CDINSCESTA = I.CDINSCESTA AND
                               F.CDEMPRESA  = I.CDEMPRESA  AND
                               I.CDPAIS   = P.CDPAIS AND
                               I.SGESTADO  = E.SGESTADO AND   
                               I.CDMUNICIPIO = M.CDMUNICIPIO AND
                               P.CDPAIS = M.CDPAIS AND
                               E.CDPAIS = P.CDPAIS AND
                               E.SGESTADO = M.SGESTADO) FATURAMENTO
                    WHERE
                          (INSTR(:L_CDFILDEST, AF.CDFILDEST) <> 0) AND
                          (INSTR(:L_NRAUTOFORN, AF.NRAUTOFORN) <> 0) AND
                          (INSTR(:L_CDFILLANC, AF.CDFILLANC) <> 0) AND
                          (AF.CDFILDEST   = PARCMPSUP.CDFILIAL)   AND
                          (AF.CDFILDEST   = ITEMAUTF.CDFILDEST)  AND
                          (AF.CDFILLANC   = ITEMAUTF.CDFILLANC)  AND
                          (AF.NRAUTOFORN  = ITEMAUTF.NRAUTOFORN) AND
                          (ITEMAUTF.CDPRODUTO = PRODUTO.CDPRODUTO) AND
                          (ITEMAUTF.CDMARCA    = MARCA.CDMARCA) AND     
                          (AF.CDFILDEST = PARAMFILIAL.CDFILIAL(+)) AND
                          (ITEMAUTF.CDSEGMARCA = SEGMARCA.CDMARCA (+)) AND
                          (AF.CDOPERADOR = OP.CDOPERADOR) AND
                          (AF.CDOPERLANCAF = OPLANC.CDOPERADOR (+)) AND
                          (AF.CDFILDEST  = FD.CDFILIAL)   AND
                          (AF.CDFORNECED = FN.CDFORNECED) AND
                          (AF.IDSITUATUAAF IS NULL)       AND
                          (AF.IDAFFORALIM = 'N')          AND
                          (NVL(AF.IDAFOPERLOG, '2') = '2')        AND 
                          (NVL(AF.IDAFPRODWEB, 'N') ='N' OR AF.IDBAIXAAF IN ('2', '3')) AND
                          (AF.CDFILDEST            = CLIENFILIAL.CDFILIAL(+))       AND
                          (CLIENFILIAL.CDCLIENTE   = CLIENTE.CDCLIENTE(+))          AND
                          (CLIENFILIAL.CDCLIENTE   = ENDECLIE.CDCLIENTE(+))         AND
                          (CLIENFILIAL.IDCLIENPRIN(+) = 'S')                        AND
                          (ENDECLIE.IDTPENDECLIE(+)   = 'P')                        AND
                          (ENDECLIE.CDPAIS         = MUNICLIE.CDPAIS(+))            AND
                          (ENDECLIE.SGESTADO       = MUNICLIE.SGESTADO(+))          AND
                          (ENDECLIE.CDMUNICIPIO    = MUNICLIE.CDMUNICIPIO(+))       AND
                          (AF.CDEMPRESA          = EMPRESA.CDEMPRESA)               AND
                          (EMPRESA.CDPAIS        = MUNICIPIO.CDPAIS)                AND
                          (EMPRESA.SGESTADO      = MUNICIPIO.SGESTADO)              AND
                          (EMPRESA.CDMUNICIPIO   = MUNICIPIO.CDMUNICIPIO)           AND
                          (AF.NRSEQENVIOAFORN    = EA.NRSEQENVIOAFORN)              AND
                          ((AF.CDFORNECED = EA.CDFORNECED) OR (FN.CDFORNERELACAF = EA.CDFORNECED)) AND
                          (FATURAMENTO.CDFILIAL  = AF.CDFILDEST)                    AND
                          (FD.CDFILCENTCMP       = PARAMGERALECOMMERCE.CDFILIAL(+)) AND 
                          (NVL(AF.IDBAIXAAF,'0') <> '6')
                    ORDER BY AF.NRSEQAPURAF, AF.NRAUTOFORN, ITEMAUTF.CDPRODUTO, ITEMAUTF.CDMARCA, ITEMAUTF.DTPROGENTAF";
        
        $params = array(
            "P_CDFORNECED" => $row['CDFORNECED'],
            "L_CDFILLANC" => $row['CDFILLANC'],
            "L_CDFILDEST" => $row['CDFILDEST'],
            "L_NRAUTOFORN" => $row['NRAUTOFORN']
        );
        
        $std = $this->connection->prepare($query);
        $std->execute($params);

        return $std->fetchAll();
    }
}